@@ -24,12 +24,13 @@ import("core.base.hashset")
 import("core.base.scheduler")
 import("core.project.project")
 import("core.base.tty")
-import("private.async.runjobs")
+import("async.runjobs")
 import("utils.progress")
 import("net.fasturl")
 import("private.action.require.impl.package")
 import("private.action.require.impl.lock_packages")
 import("private.action.require.impl.register_packages")
+import("private.action.require.impl.actions.check", {alias = "action_check"})
 import("private.action.require.impl.actions.install", {alias = "action_install"})
 import("private.action.require.impl.actions.download", {alias = "action_download"})
 
@@ -357,6 +358,14 @@ function _fetch_packages(packages_fetch, installdeps)
           isolate = true})
 end
 
+-- should fetch package?
+function _should_fetch_package(instance)
+    if instance:is_fetchonly() or not option.get("force") or
+        (option.get("shallow") and not instance:is_toplevel()) then
+        return true
+    end
+end
+
 -- should install package?
 function _should_install_package(instance)
     _g.package_status_cache = _g.package_status_cache or {}
@@ -371,7 +380,7 @@ end
 -- install packages
 function _install_packages(packages_install, packages_download, installdeps)
 
-    -- we need hide wait characters if is not a tty
+    -- we need to hide wait characters if is not a tty
     local show_wait = io.isatty()
 
     -- init installed packages
@@ -469,6 +478,7 @@ function _install_packages(packages_install, packages_download, installdeps)
                 local downloaded = true
                 if packages_download[tostring(instance)] then
                     packages_downloading[index] = instance
+                    action_check(instance)
                     downloaded = action_download(instance)
                     packages_downloading[index] = nil
                 end
@@ -478,8 +488,9 @@ function _install_packages(packages_install, packages_download, installdeps)
                 if downloaded then
                     if not action_install(instance) then
                         assert(instance:is_precompiled(), "package(%s) should be precompiled", instance:name())
-                        -- we need disable built and re-download and re-install it
+                        -- we need to disable built and re-download and re-install it
                         instance:fallback_build()
+                        action_check(instance)
                         action_download(instance)
                         action_install(instance)
                     end
@@ -628,7 +639,7 @@ function _get_package_installdeps(packages)
     local installdeps = {}
     local packagesmap = {}
     for _, instance in ipairs(packages) do
-        -- we need use alias name first for toolchain/packages
+        -- we need to use alias name first for toolchain/packages
         packagesmap[instance:alias() or instance:name()] = instance
     end
     for _, instance in ipairs(packages) do
@@ -636,7 +647,7 @@ function _get_package_installdeps(packages)
         if instance:orderdeps() then
             deps = table.copy(instance:orderdeps())
         end
-        -- patch toolchain/packages to installdeps, because we need install toolchain package first
+        -- patch toolchain/packages to installdeps, because we need to install toolchain package first
         for _, toolchain in ipairs(instance:toolchains()) do
             for _, packagename in ipairs(toolchain:config("packages")) do
                 if packagesmap[packagename] ~= instance then -- avoid loop recursion
@@ -672,9 +683,7 @@ function main(requires, opt)
     -- fetch and register packages (with system) from local first
     local packages_fetch = {}
     for _, instance in ipairs(packages) do
-        if instance and (instance:is_fetchonly() or
-                         not option.get("force") or
-                         (option.get("shallow") and not instance:is_toplevel())) then
+        if _should_fetch_package(instance) then
             table.insert(packages_fetch, instance)
         end
     end
@@ -733,7 +742,11 @@ function main(requires, opt)
 
     -- exists not found packages?
     if #packages_not_found > 0 then
-        cprint("${bright color.warning}note: ${clear}the following packages were not found on your system, try again after installing them:")
+        if packages_not_found[1]:is_cross() then
+            cprint("${bright color.warning}note: ${clear}system package is not supported for cross-compilation currently, the following system packages cannot be found:")
+        else
+            cprint("${bright color.warning}note: ${clear}the following packages were not found on your system, try again after installing them:")
+        end
         for _, instance in ipairs(packages_not_found) do
             print("  -> %s %s", instance:displayname(), instance:version_str() or "")
         end
@@ -766,7 +779,7 @@ function main(requires, opt)
         print("upgrading packages ..")
     end
 
-    -- some packages are modified? we need fix packages list and all deps
+    -- some packages are modified? we need to fix packages list and all deps
     if packages_modified then
         order_packages = {}
         _replace_packages(packages, packages_modified)
